#include "dxsoftex.h"
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>


volatile bool quit = false;

void sigint_handler( int sig )
{
	quit = true;
}

void print_usage()
{
	printf( "Usage: dxlog [-escape] [-escapefile] [-escapetty] [-append] [-out <logfile>] [-channel <number>]\n" );

	printf( "\t -escape\tEscape TTY and File output\n" );
	printf( "\t -escapefile\tEscape File output\n" );
	printf( "\t -escapetty\tEscape TTY output\n" );
	printf( "\t -append\tAppend to output file\n" );
	printf( "\t -out\t\tSpecify output file\n" );
	printf( "\t -channel\tSelect channel\n" );
	printf( "\t -help\t\tThis output\n" );

	exit( -1 );
}

int main( int argc, char *argv[] )
{
	int channelNumber = 1;
	bool escapeLog = false;
	bool escapeTTY = false;
	bool append = false;
	const char *filename = "";
	FILE *fp = NULL;

	int arg = 1;

	while( arg < argc )
	{
		if( !stricmp( argv[arg], "-escape" ) )
		{
			escapeTTY = true;
			escapeLog = true;
			arg++;
		}
		else if( !stricmp( argv[arg], "-escapefile" ) )
		{
			escapeLog = true;
			arg++;
		}
		else if( !stricmp( argv[arg], "-escapetty" ) )
		{
			escapeTTY = true;
			arg++;
		}
		else if( !stricmp( argv[arg], "-append" ) )
		{
			append = true;
			arg++;
		}
		else if( !stricmp( argv[arg], "-help" ) )
		{
			print_usage();
		}
		else if( arg + 1 < argc )
		{
			if( !stricmp( argv[arg], "-out" ) )
			{
				filename = argv[arg + 1];
				arg += 2;
			}
			else if( !stricmp( argv[arg], "-channel" ) )
			{
				channelNumber = atoi( argv[arg + 1] );
				arg += 2;
			}
			else
				print_usage();
		}
		else
			print_usage();
	}


	if( !DxsoftExchangeInit( channelNumber ) )
	{
		printf( "DxsoftExchangeInit(%d) failed\n.", channelNumber );
		return -1;
	}

	signal( SIGINT, sigint_handler );

	if( filename[0] != 0 )
	{
		if( append )
			fp = fopen( filename, "ab" );
		else
			fp = fopen( filename, "wb" );

		if( fp == NULL )
		{
			printf( "Could not open '%s' for writing.\n", filename );
			return -1;
		}
	}

	while( !quit )
	{
		int out = DxsoftGetSymFromRxBuffer();
		if( out != -1 )
		{
			char raw[2];
			char escaped[16];

			raw[0] = (char)out;
			raw[1] = 0;
			sprintf( escaped, "[#%03u]", out );

			if( fp )
			{
				if( escapeLog && ( out < 32 || out >= 128 ) )
					fputs( escaped, fp );
				else
					fputs( raw, fp );

				fflush( fp );
			}

			if( escapeTTY && ( out < 32 || out >= 128 ) )
				fputs( escaped, stdout );
			else
				fputs( raw, stdout );

		}
		else
		{
			Sleep( 100 );
		}
	}

	printf( "\nGot quit signal, exiting...\n" );

	if( fp )
		fclose( fp );

	return 0;
}

