//---------------------------------------------------------------------------
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tchar.h>
#pragma hdrstop
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
#define RX_BUFF_SIZE 2048
#define TX_BUFF_SIZE 2048


 struct Exchange_Struct
  {
   unsigned char RxBuffer[RX_BUFF_SIZE];
   unsigned char TxBuffer[TX_BUFF_SIZE];
   unsigned int RxHead;
   unsigned int RxTail;
   unsigned int TxHead;
   unsigned int TxTail;
   unsigned int Signature;
  } ;

 struct Exchange_Struct *Exch;

 static LPVOID lpvMem = NULL; // pointer to shared memory
 static HANDLE hMapObject = NULL;  // handle to file mapping
 static int IsInited=0; // Init flag

 //---------------------------------------------------------------------------
int WINAPI DxsoftExchangeShutdown()
{
	if(lpvMem!=NULL)
		UnmapViewOfFile(lpvMem);
    if(hMapObject!=NULL)
		CloseHandle(hMapObject);
    return TRUE;
}


//---------------------------------------------------------------------------
// This function is called by both programs
int DxsoftExchangeInit(int canal_number)
{
    BOOL fInit;
    static wchar_t str[40];

 //  " "   
 //  TRUE  , FALSE -  


    // Unmap shared memory from the process's address space.
    if(lpvMem!=NULL) {UnmapViewOfFile(lpvMem);lpvMem=NULL;}
    // Close the process's handle to the file-mapping object.
    if(hMapObject!=NULL) {CloseHandle(hMapObject);hMapObject=NULL;}
    IsInited=0;

 // Create a named file mapping object.

    wsprintf(str, _T( "dxsoftexchange-filemap-%x" ),canal_number);

    hMapObject = CreateFileMapping(
        (HANDLE) 0xFFFFFFFF, // use paging file
        NULL,                // no security attributes
        PAGE_READWRITE,      // read/write access
        0,                   // size: high 32-bits
        sizeof(struct Exchange_Struct),           // size: low 32-bits
        str);    // name of map object
        if (hMapObject == NULL)
        return FALSE;

    // The first process to attach initializes memory.

    fInit = (GetLastError() != ERROR_ALREADY_EXISTS);

    // Get a pointer to the file-mapped shared memory.

    lpvMem = MapViewOfFile(

        hMapObject,     // object to map view of
        FILE_MAP_WRITE, // read/write access
        0,              // high offset:  map from
        0,              // low offset:   beginning
        0);             // default: map entire file
    if (lpvMem == NULL)
        return FALSE;

    // Initialize memory if this is the first process.

    if (fInit)
       {
        memset(lpvMem, '\0', sizeof(struct Exchange_Struct));
        Exch= (struct Exchange_Struct*) lpvMem;
        Exch->RxHead=0;
        Exch->RxTail=0;
        Exch->TxHead=0;
        Exch->TxTail=0;
       }
    else
       {
        Exch= (struct Exchange_Struct*) lpvMem;
       }

  IsInited=1;
  return TRUE;
}
//---------------------------------------------------------------------------
// There are functions called by external software
//---------------------------------------------------------------------------
int DxsoftGetSymFromRxBuffer(void)
 {
  // returns received symbol or -1,
  // if buffer is empty
  int sym;

  if(!IsInited) return -1;

  if(Exch->RxHead == Exch->RxTail) return -1;

  sym=Exch->RxBuffer[Exch->RxTail]&255;
  Exch->RxTail=  (Exch->RxTail+1)&(RX_BUFF_SIZE-1);
  return sym;
 }

//---------------------------------------------------------------------------
int DxsoftPutSymToTxBuffer(char sym)
 {
  // Puts symbol to TX buffer
  // return 1 if OK,
  // 0 if ERROR (buffer is full)
  int len;

  if(!IsInited) return 0;

  len =  (Exch->TxHead - Exch->TxTail)&(TX_BUFF_SIZE-1);
  if(len>(TX_BUFF_SIZE-4)) return 0;

  Exch->TxBuffer[Exch->TxHead]=sym;
  Exch->TxHead=  (Exch->TxHead+1)&(TX_BUFF_SIZE-1);

  return 1;
 }

//---------------------------------------------------------------------------
// There are functions, called by TrueTTY
//---------------------------------------------------------------------------
int DxsoftGetSymFromTxBuffer(void)
 {
  // Returns symbol from TX buffer
  // Returns -1 if buffer is empty
  int sym;

  if(!IsInited) return -1;

  if(Exch->TxHead == Exch->TxTail) return -1;

  sym=Exch->TxBuffer[Exch->TxTail]&255;
  Exch->TxTail=  (Exch->TxTail+1)&(TX_BUFF_SIZE-1);
  return sym;
 }
//---------------------------------------------------------------------------
int DxsoftPutSymToRxBuffer(char sym)
 {
  // Puts symbol to RX buffer
  // Returns 1 if OK,
  // returns 0 if error (buffer is full)
  int len;

  if(!IsInited) return 0;

  len =  (Exch->RxHead - Exch->RxTail)&(RX_BUFF_SIZE-1);
  if(len>(RX_BUFF_SIZE-4)) return 0;

  Exch->RxBuffer[Exch->RxHead]=sym;
  Exch->RxHead=  (Exch->RxHead+1)&(RX_BUFF_SIZE-1);

  return 1;
 }
//---------------------------------------------------------------------------


